import React from 'react';
import { useTranslation } from 'react-i18next';
import { Star, TrendingUp, Clock, Compass, Eye, Telescope, Sparkles, Navigation } from 'lucide-react';
import { StarConditions } from '../types';
import { getIndividualScores } from '../utils/calculations';

interface StarScoreProps {
  score: number;
  loading?: boolean;
  conditions?: StarConditions;
  selectedTime?: Date;
}

const StarScore: React.FC<StarScoreProps> = ({ score, loading, conditions, selectedTime }) => {
  const { t } = useTranslation();

  const getScoreDescription = (score: number): string => {
    if (score >= 4.5) return t('score.excellent');
    if (score >= 3.5) return t('score.good');
    if (score >= 2.5) return t('score.fair');
    if (score >= 1.5) return t('score.poor');
    return t('score.bad');
  };

  const getScoreColor = (score: number): string => {
    if (score >= 4) return 'from-green-500 to-emerald-600';
    if (score >= 3) return 'from-yellow-500 to-amber-600';
    if (score >= 2) return 'from-orange-500 to-red-500';
    return 'from-red-500 to-red-700';
  };

  const renderStars = () => {
    return Array.from({ length: 5 }, (_, i) => (
      <Star
        key={i}
        className={
          'h-6 w-6 transition-all duration-300 ' +
          (i < Math.floor(score) 
            ? 'text-yellow-400 fill-current animate-twinkle' 
            : i < score 
            ? 'text-yellow-400 fill-current opacity-50'
            : 'text-gray-500')
        }
        style={{ animationDelay: `${i * 0.1}s` }}
      />
    ));
  };

  // Calculate individual scores with updated weights
  const currentTime = selectedTime || new Date();
  const individualScores = conditions?.weather && conditions?.lightPollution ? 
    getIndividualScores(conditions.weather, conditions.lightPollution, currentTime) : null;

  // Calculate visibility forecast for next 24 hours from current time
  const getHourlyForecast = () => {
    if (!conditions?.weather || !conditions?.lightPollution) return [];
    
    const forecast = [];
    const now = selectedTime || new Date();
    
    for (let i = 0; i < 24; i++) {
      const hour = new Date(now.getTime() + i * 60 * 60 * 1000);
      const scores = getIndividualScores(conditions.weather, conditions.lightPollution, hour);
      const hourScore = (
        scores.weatherScore * 0.30 + 
        scores.lightPollutionScore * 0.20 + 
        scores.moonScore * 0.10 + 
        scores.timeScore * 0.40
      );
      
      forecast.push({
        time: hour.toLocaleTimeString('ja-JP', { hour: '2-digit', minute: '2-digit' }),
        score: Math.min(5, hourScore),
        hour: hour.getHours(),
        isPeak: hourScore >= 4
      });
    }
    
    return forecast;
  };

  // Enhanced visible constellations based on location and time
  const getVisibleConstellations = () => {
    const month = (selectedTime || new Date()).getMonth();
    const hour = (selectedTime || new Date()).getHours();
    
    const allConstellations = [
      { 
        name: t('constellations.orion.name') || 'オリオン座', 
        direction: t('constellations.orion.direction') || '南', 
        season: [11, 0, 1, 2], 
        brightness: '★★★★★',
        stars: ['ベテルギウス', 'リゲル', 'ベラトリックス', 'ミンタカ', 'アルニラム', 'アルニタク'],
        magnitude: 1.6,
        description: t('constellations.orion.description') || '冬の代表的な星座。三つ星が目印。',
        mythology: t('constellations.orion.mythology') || 'ギリシャ神話の狩人オリオンの姿',
        bestTime: '21:00-02:00',
        icon: '🏹'
      },
      { 
        name: t('constellations.cassiopeia.name') || 'カシオペア座', 
        direction: t('constellations.cassiopeia.direction') || '北', 
        season: [9, 10, 11, 0, 1, 2], 
        brightness: '★★★★☆',
        stars: ['シェダル', 'カフ', 'ガンマ・カス', 'ルクバー', 'セギン'],
        magnitude: 2.2,
        description: t('constellations.cassiopeia.description') || 'W字型の特徴的な形。北極星を見つける目印。',
        mythology: t('constellations.cassiopeia.mythology') || 'エチオピア王妃カシオペアの姿',
        bestTime: '20:00-05:00',
        icon: '👑'
      },
      { 
        name: t('constellations.ursaMajor.name') || 'おおぐま座', 
        direction: t('constellations.ursaMajor.direction') || '北', 
        season: [2, 3, 4, 5, 6], 
        brightness: '★★★★☆',
        stars: ['ドゥーベ', 'メラク', 'フェクダ', 'メグレズ', 'アリオト', 'ミザール', 'ベネトナシュ'],
        magnitude: 1.8,
        description: t('constellations.ursaMajor.description') || '北斗七星を含む大きな星座。春の夜空の主役。',
        mythology: t('constellations.ursaMajor.mythology') || '大熊の姿、北斗七星は柄杓の形',
        bestTime: '21:00-04:00',
        icon: '🐻'
      },
      { 
        name: t('constellations.cygnus.name') || 'はくちょう座', 
        direction: t('constellations.cygnus.direction') || '天頂', 
        season: [6, 7, 8, 9, 10], 
        brightness: '★★★★☆',
        stars: ['デネブ', 'アルビレオ', 'サドル', 'ギエナー', 'デルタ・シグニ'],
        magnitude: 1.3,
        description: t('constellations.cygnus.description') || '夏の大三角の一角。十字の形が美しい。',
        mythology: t('constellations.cygnus.mythology') || '白鳥の姿、十字架とも呼ばれる',
        bestTime: '21:00-03:00',
        icon: '🦢'
      },
      { 
        name: t('constellations.scorpius.name') || 'さそり座', 
        direction: t('constellations.scorpius.direction') || '南', 
        season: [5, 6, 7, 8, 9], 
        brightness: '★★★★★',
        stars: ['アンタレス', 'シャウラ', 'サルガス', 'デシュビア', 'ピー・スコルピイ'],
        magnitude: 1.1,
        description: t('constellations.scorpius.description') || '赤い巨星アンタレスが印象的。夏の南天の王者。',
        mythology: t('constellations.scorpius.mythology') || 'オリオンを刺したサソリの姿',
        bestTime: '21:00-01:00',
        icon: '🦂'
      },
      { 
        name: t('constellations.lyra.name') || 'こと座', 
        direction: t('constellations.lyra.direction') || '天頂', 
        season: [6, 7, 8, 9], 
        brightness: '★★★☆☆',
        stars: ['ベガ', 'シェリアク', 'スラファト', 'エータ・リラエ'],
        magnitude: 0.0,
        description: t('constellations.lyra.description') || '織姫星ベガを含む小さな星座。',
        mythology: t('constellations.lyra.mythology') || 'オルフェウスの竪琴の姿',
        bestTime: '21:00-02:00',
        icon: '🎵'
      },
      { 
        name: t('constellations.aquila.name') || 'わし座', 
        direction: t('constellations.aquila.direction') || '南東', 
        season: [7, 8, 9, 10], 
        brightness: '★★★☆☆',
        stars: ['アルタイル', 'タラゼド', 'アルシャイン', 'ベザ'],
        magnitude: 0.8,
        description: t('constellations.aquila.description') || '彦星アルタイルを含む。夏の大三角の一角。',
        mythology: t('constellations.aquila.mythology') || 'ゼウスの鷲の姿',
        bestTime: '20:00-01:00',
        icon: '🦅'
      },
      { 
        name: t('constellations.pegasus.name') || 'ペガスス座', 
        direction: t('constellations.pegasus.direction') || '南', 
        season: [9, 10, 11, 0], 
        brightness: '★★★☆☆',
        stars: ['マルカブ', 'シェアト', 'アルゲニブ', 'エニフ'],
        magnitude: 2.5,
        description: t('constellations.pegasus.description') || '秋の四辺形で有名。大きな正方形が目印。',
        mythology: t('constellations.pegasus.mythology') || '天馬ペガサスの姿',
        bestTime: '21:00-03:00',
        icon: '🐴'
      }
    ];

    // Filter by season and return more constellations
    return allConstellations.filter(c => c.season.includes(month)).slice(0, 8);
  };

  // Calculate estimated visible stars - ADJUSTED TO 1000 IDEAL
  const getVisibleStarCount = () => {
    if (!individualScores) return 500;
    
    const baseCount = 1000; // CHANGED: Ideal conditions now 1000 stars
    let visibleCount = baseCount;
    
    // More realistic calculation based on Bortle scale and conditions
    const lightPollutionFactor = Math.max(0.05, Math.pow(individualScores.lightPollutionScore / 5, 1.5));
    visibleCount *= lightPollutionFactor;
    
    // Weather impact
    const weatherFactor = Math.max(0.1, Math.pow(individualScores.weatherScore / 5, 2));
    visibleCount *= weatherFactor;
    
    // Moon impact
    const moonFactor = Math.max(0.4, 1 - (1 - individualScores.moonScore / 5) * 0.6);
    visibleCount *= moonFactor;
    
    // Time factor (atmospheric conditions)
    const timeFactor = Math.max(0.7, individualScores.timeScore / 5);
    visibleCount *= timeFactor;
    
    return Math.round(Math.max(50, visibleCount));
  };

  const visibleConstellations = getVisibleConstellations();
  const hourlyForecast = getHourlyForecast();
  const visibleStarCount = getVisibleStarCount();
  const bestHours = hourlyForecast.filter(f => f.isPeak);

  if (loading) {
    return (
      <div className="bg-slate-900/50 backdrop-blur-sm rounded-2xl p-6 border border-slate-700/50 animate-pulse">
        <div className="h-8 bg-slate-700/50 rounded mb-4"></div>
        <div className="h-16 bg-slate-700/50 rounded mb-4"></div>
        <div className="h-4 bg-slate-700/50 rounded"></div>
      </div>
    );
  }

  return (
    <div className="bg-slate-900/50 backdrop-blur-sm rounded-2xl p-6 border border-slate-700/50 animate-slide-up">
      <div className="flex items-center justify-between mb-4">
        <h3 className="text-lg font-semibold text-white flex items-center">
          <TrendingUp className="h-5 w-5 mr-2" />
          {t('starScore.title')}
        </h3>
      </div>

      <div className="text-center mb-6">
        <div className={'inline-flex items-center justify-center w-24 h-24 rounded-full bg-gradient-to-br ' + getScoreColor(score) + ' mb-4 shadow-lg relative animate-float'}>
          <span className="text-3xl font-bold text-white">
            {score.toFixed(1)}
          </span>
          {score >= 4 && (
            <div className="absolute -top-2 -right-2">
              <Sparkles className="h-6 w-6 text-yellow-300 animate-pulse" />
            </div>
          )}
        </div>
        
        <div className="flex justify-center space-x-1 mb-2">
          {renderStars()}
        </div>
        
        <p className="text-white font-medium">
          {getScoreDescription(score)}
        </p>
        <p className="text-gray-400 text-sm mt-1">
          {t('starScore.subtitle')}
        </p>
      </div>

      {/* Detailed breakdown with updated weights */}
      {individualScores && (
        <div className="bg-slate-800/50 rounded-lg p-4 mb-4 border border-slate-700/30 animate-fade-in">
          <h4 className="text-sm font-medium text-white mb-3">{t('starScore.factors')}</h4>
          <div className="space-y-3">
            <div className="flex justify-between items-center">
              <span className="text-gray-300 text-sm">{t('starScore.weather')} (30%)</span>
              <div className="flex items-center space-x-2">
                <div className="w-16 bg-slate-700 rounded-full h-2">
                  <div 
                    className="h-2 rounded-full bg-blue-400 transition-all duration-1000 ease-out" 
                    style={{ width: `${(individualScores.weatherScore / 5) * 100}%` }}
                  ></div>
                </div>
                <span className="text-white text-sm w-8">{individualScores.weatherScore.toFixed(1)}</span>
              </div>
            </div>
            
            <div className="flex justify-between items-center">
              <span className="text-gray-300 text-sm">{t('starScore.lightPollution')} (20%)</span>
              <div className="flex items-center space-x-2">
                <div className="w-16 bg-slate-700 rounded-full h-2">
                  <div 
                    className="h-2 rounded-full bg-orange-400 transition-all duration-1000 ease-out" 
                    style={{ width: `${(individualScores.lightPollutionScore / 5) * 100}%`, animationDelay: '0.2s' }}
                  ></div>
                </div>
                <span className="text-white text-sm w-8">{individualScores.lightPollutionScore.toFixed(1)}</span>
              </div>
            </div>
            
            <div className="flex justify-between items-center">
              <span className="text-gray-300 text-sm">{t('starScore.moonPhase')} (10%)</span>
              <div className="flex items-center space-x-2">
                <div className="w-16 bg-slate-700 rounded-full h-2">
                  <div 
                    className="h-2 rounded-full bg-purple-400 transition-all duration-1000 ease-out" 
                    style={{ width: `${(individualScores.moonScore / 5) * 100}%`, animationDelay: '0.4s' }}
                  ></div>
                </div>
                <span className="text-white text-sm w-8">{individualScores.moonScore.toFixed(1)}</span>
              </div>
            </div>
            
            <div className="flex justify-between items-center">
              <span className="text-gray-300 text-sm">{t('starScore.time')} (40%)</span>
              <div className="flex items-center space-x-2">
                <div className="w-16 bg-slate-700 rounded-full h-2">
                  <div 
                    className="h-2 rounded-full bg-green-400 transition-all duration-1000 ease-out" 
                    style={{ width: `${(individualScores.timeScore / 5) * 100}%`, animationDelay: '0.6s' }}
                  ></div>
                </div>
                <span className="text-white text-sm w-8">{individualScores.timeScore.toFixed(1)}</span>
              </div>
            </div>
          </div>
        </div>
      )}

      {/* Visible star count - UPDATED TO 1000 IDEAL */}
      <div className="bg-slate-800/50 rounded-lg p-4 mb-4 border border-slate-700/30 animate-fade-in" style={{ animationDelay: '0.2s' }}>
        <h4 className="text-sm font-medium text-white mb-3 flex items-center">
          <Eye className="h-4 w-4 mr-2" />
          {t('starScore.visibleStars')}
        </h4>
        <div className="flex items-center justify-between">
          <div className="flex items-center space-x-3">
            <div className="text-2xl font-bold text-yellow-400 animate-pulse-slow">
              {visibleStarCount.toLocaleString()}
            </div>
            <div className="text-sm text-gray-300">{t('starScore.starsUnit')}</div>
          </div>
          <div className="text-xs text-gray-400">
            {t('starScore.idealConditions')}: 1,000{t('starScore.starsUnit')}
          </div>
        </div>
        <div className="mt-2 w-full bg-slate-700 rounded-full h-2">
          <div 
            className="h-2 rounded-full bg-gradient-to-r from-yellow-400 to-orange-400 transition-all duration-2000 ease-out" 
            style={{ width: `${(visibleStarCount / 1000) * 100}%` }}
          ></div>
        </div>
      </div>

      {/* Enhanced visible constellations */}
      <div className="bg-slate-800/50 rounded-lg p-4 mb-4 border border-slate-700/30 animate-fade-in" style={{ animationDelay: '0.4s' }}>
        <h4 className="text-sm font-medium text-white mb-3 flex items-center">
          <Compass className="h-4 w-4 mr-2" />
          {t('starScore.visibleConstellations')} ({visibleConstellations.length}{t('starScore.constellationsUnit')})
        </h4>
        <div className="space-y-3 max-h-80 overflow-y-auto">
          {visibleConstellations.map((constellation, index) => (
            <div 
              key={index} 
              className="bg-slate-700/30 rounded-lg p-3 hover:bg-slate-700/50 transition-all duration-300 hover:scale-105 animate-slide-up"
              style={{ animationDelay: `${index * 0.1}s` }}
            >
              <div className="flex justify-between items-start mb-2">
                <div className="flex items-center space-x-2">
                  <span className="text-lg animate-bounce-gentle" style={{ animationDelay: `${index * 0.2}s` }}>
                    {constellation.icon}
                  </span>
                  <span className="text-white font-medium">{constellation.name}</span>
                  <span className="text-xs text-gray-400 bg-slate-600/50 px-2 py-1 rounded flex items-center">
                    <Navigation className="h-3 w-3 mr-1" />
                    {constellation.direction}
                  </span>
                </div>
                <div className="text-xs text-yellow-400">{constellation.brightness}</div>
              </div>
              
              <div className="text-xs text-gray-300 mb-2">{constellation.description}</div>
              <div className="text-xs text-purple-300 mb-2 italic">{constellation.mythology}</div>
              
              <div className="flex items-center justify-between text-xs">
                <div className="flex items-center space-x-2">
                  <Telescope className="h-3 w-3 text-purple-400" />
                  <div className="text-purple-300">
                    {t('starScore.mainStars')}: {constellation.stars.slice(0, 3).join(', ')}
                    {constellation.stars.length > 3 && ` +${constellation.stars.length - 3}${t('starScore.starsUnit')}`}
                  </div>
                </div>
              </div>
              
              <div className="flex items-center justify-between text-xs mt-2">
                <span className="text-gray-400">{t('starScore.magnitude')}: {constellation.magnitude}</span>
                <span className="text-green-400">{t('starScore.bestTime')}: {constellation.bestTime}</span>
              </div>
            </div>
          ))}
        </div>
      </div>

      {/* 24-hour forecast with enhanced visualization - WIDER CHART */}
      <div className="bg-slate-800/50 rounded-lg p-4 border border-slate-700/30 animate-fade-in" style={{ animationDelay: '0.6s' }}>
        <h4 className="text-sm font-medium text-white mb-3 flex items-center">
          <Clock className="h-4 w-4 mr-2" />
          {t('timeControls.24hourForecast')}
        </h4>
        
        {/* Line chart visualization - EXPANDED */}
        <div className="relative h-40 mb-4 bg-slate-900/30 rounded-lg p-3 overflow-x-auto">
          <svg className="w-full h-full min-w-[600px]" viewBox="0 0 600 150">
            {/* Grid lines */}
            {[0, 1, 2, 3, 4, 5].map(score => (
              <line
                key={score}
                x1="0"
                y1={150 - (score * 30)}
                x2="600"
                y2={150 - (score * 30)}
                stroke="#374151"
                strokeWidth="0.5"
                strokeDasharray="2,2"
              />
            ))}
            
            {/* Vertical time grid lines */}
            {Array.from({ length: 25 }, (_, i) => (
              <line
                key={`vgrid-${i}`}
                x1={i * 25}
                y1="0"
                x2={i * 25}
                y2="150"
                stroke="#374151"
                strokeWidth="0.3"
                strokeOpacity="0.3"
              />
            ))}
            
            {/* Gradient definition */}
            <defs>
              <linearGradient id="scoreGradient" x1="0%" y1="0%" x2="0%" y2="100%">
                <stop offset="0%" stopColor="#8b5cf6" stopOpacity="0.8"/>
                <stop offset="100%" stopColor="#8b5cf6" stopOpacity="0.1"/>
              </linearGradient>
            </defs>
            
            {/* Area under curve */}
            <polygon
              points={`0,150 ${hourlyForecast.map((forecast, index) => 
                `${index * 25},${150 - (forecast.score * 30)}`
              ).join(' ')} 600,150`}
              fill="url(#scoreGradient)"
            />
            
            {/* Score line */}
            <polyline
              points={hourlyForecast.map((forecast, index) => 
                `${index * 25},${150 - (forecast.score * 30)}`
              ).join(' ')}
              fill="none"
              stroke="#8b5cf6"
              strokeWidth="3"
              className="drop-shadow-sm"
            />
            
            {/* Data points with score values */}
            {hourlyForecast.map((forecast, index) => (
              <g key={index}>
                <circle
                  cx={index * 25}
                  cy={150 - (forecast.score * 30)}
                  r="4"
                  fill={forecast.isPeak ? '#f59e0b' : '#8b5cf6'}
                  stroke="white"
                  strokeWidth="1"
                  className="drop-shadow-sm animate-pulse-slow"
                  style={{ animationDelay: `${index * 0.1}s` }}
                />
                
                {/* Score value labels */}
                <text
                  x={index * 25}
                  y={150 - (forecast.score * 30) - 8}
                  textAnchor="middle"
                  fontSize="8"
                  fill={forecast.isPeak ? '#f59e0b' : '#8b5cf6'}
                  className="font-bold"
                >
                  {forecast.score.toFixed(1)}
                </text>
              </g>
            ))}
            
            {/* Peak indicators */}
            {hourlyForecast.map((forecast, index) => 
              forecast.isPeak ? (
                <g key={`peak-${index}`}>
                  <text
                    x={index * 25}
                    y={150 - (forecast.score * 30) - 20}
                    textAnchor="middle"
                    fontSize="12"
                    fill="#f59e0b"
                    className="font-bold animate-pulse"
                  >
                    ★
                  </text>
                </g>
              ) : null
            )}
            
            {/* Current time indicator */}
            <line
              x1="0"
              y1="0"
              x2="0"
              y2="150"
              stroke="#f59e0b"
              strokeWidth="3"
              strokeDasharray="6,3"
            />
            <text
              x="5"
              y="15"
              fontSize="10"
              fill="#f59e0b"
              className="font-bold"
            >
              {t('timeControls.currentTime')}
            </text>
          </svg>
          
          {/* Y-axis labels */}
          <div className="absolute left-0 top-0 h-full flex flex-col justify-between text-xs text-gray-400 -ml-10">
            <span>5.0</span>
            <span>4.0</span>
            <span>3.0</span>
            <span>2.0</span>
            <span>1.0</span>
            <span>0.0</span>
          </div>
        </div>
        
        {/* X-axis labels - CORRECTED TO MATCH GRAPH */}
        <div className="grid grid-cols-6 text-xs text-gray-400 mb-3 px-3">
          <span className="text-center">{t('timeControls.currentTime')}</span>
          <span className="text-center">4{t('timeControls.hoursLater')}</span>
          <span className="text-center">8{t('timeControls.hoursLater')}</span>
          <span className="text-center">12{t('timeControls.hoursLater')}</span>
          <span className="text-center">16{t('timeControls.hoursLater')}</span>
          <span className="text-center">20{t('timeControls.hoursLater')}</span>
        </div>
        
        {/* Detailed time labels */}
        <div className="flex justify-between text-xs text-gray-500 mb-3 px-3">
          {hourlyForecast.filter((_, index) => index % 4 === 0).map((forecast, index) => (
            <span key={index} className="text-center">
              {forecast.time}
            </span>
          ))}
        </div>
        
        {/* Best times highlight with peak scores */}
        {bestHours.length > 0 && (
          <div className="bg-green-500/10 border border-green-500/20 rounded-lg p-3 animate-pulse-slow">
            <div className="flex items-center space-x-2 mb-2">
              <div className="w-2 h-2 bg-green-400 rounded-full animate-pulse"></div>
              <span className="text-green-400 text-sm font-medium">{t('timeControls.bestTime')} ({bestHours.length}{t('timeControls.hours')})</span>
            </div>
            <div className="grid grid-cols-2 gap-2 text-xs">
              {bestHours.map((hour, index) => (
                <div key={index} className="flex justify-between text-green-300">
                  <span>{hour.time}</span>
                  <span className="font-bold">★{hour.score.toFixed(1)}</span>
                </div>
              ))}
            </div>
          </div>
        )}
      </div>
    </div>
  );
};

export default StarScore;